@@ -3,22 +3,22 @@
  * ------------------------------------------------------------------
  *  Last change: Time-stamp: <2001-04-29 12:06:48 jao>
  * ------------------------------------------------------------------
- * Copyright (C) 2001 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 
@@ -35,10 +35,6 @@
 extern gboolean
 mixgtk_init (int argc, char *argv[]);
 
-/* restart the gui */
-extern gboolean
-mixgtk_restart (void);
-
 /* enter the main mixgtk loop */
 extern void
 mixgtk_main (void);
