@@ -1,24 +1,24 @@
 /* -*-c-*- -------------- mixgtk_cmd_dispatcher.c :
  * Implementation of the functions declared in mixgtk_cmd_dispatcher.h
  * ------------------------------------------------------------------
- * $Id: mixgtk_cmd_dispatcher.c,v 1.15 2002/04/09 23:28:19 jao Exp $
+ * $Id: mixgtk_cmd_dispatcher.c,v 1.23 2004/08/01 23:30:54 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2001, 2002 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2001, 2002, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 #include <stdlib.h>
@@ -43,6 +43,7 @@
 #include <mixlib/mix_vm_command.h>
 #include "mixgtk_mixvm.h"
 #include "mixgtk_mixal.h"
+#include "mixgtk_fontsel.h"
 #include "mixgtk_config.h"
 #include "mixgtk_cmd_dispatcher.h"
 
@@ -74,9 +75,14 @@ static const gchar *TITLE_FORMAT_ = "gmixvm - %s";
 static void
 log_command_ (mixgtk_dispatch_data_t *dis, const gchar *cmd)
 {
-  gtk_text_insert (GTK_TEXT (dis->log), NULL, NULL, NULL,"MIX> ", -1);
-  gtk_text_insert (GTK_TEXT (dis->log), NULL, NULL, NULL, cmd, -1);
-  gtk_text_insert (GTK_TEXT (dis->log), NULL, NULL, NULL, "\n", -1);
+  GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dis->log));
+  GtkTextIter end;
+
+  gtk_text_buffer_get_end_iter (buf, &end);
+  gtk_text_buffer_place_cursor (buf, &end);
+  gtk_text_buffer_insert_at_cursor (buf, "MIX> ", -1);
+  gtk_text_buffer_insert_at_cursor (buf, cmd, -1);
+  gtk_text_buffer_insert_at_cursor (buf, "\n", -1);
 
 #ifdef HAVE_LIBHISTORY
   add_history ((char *)cmd);
@@ -90,16 +96,26 @@ flush_log_ (mixgtk_dispatch_data_t *dis)
 {
   enum {BLKSIZE = 100};
   static gchar BUFFER[BLKSIZE];
-  
+
+  GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dis->log));
+  GtkTextMark *mark;
+  GtkTextIter end;
   ssize_t k;
+
+  gtk_text_buffer_get_end_iter (buffer, &end);
+  gtk_text_buffer_place_cursor (buffer, &end);
+
   fflush (dis->out);
   while ((k = read (dis->fildes[0], BUFFER, BLKSIZE)) != 0)
     {
       if (k == -1 && errno != EINTR) break;
       if (k != -1)
-	gtk_text_insert (GTK_TEXT (dis->log), NULL, NULL, NULL,
-			 BUFFER, k);
+	gtk_text_buffer_insert_at_cursor (buffer, BUFFER, k);
     }
+
+  mark = gtk_text_buffer_get_insert (buffer);
+  gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (dis->log), mark, 0,
+                                TRUE, 0, 0);
 }
 
 /* hooks */
@@ -116,7 +132,7 @@ load_post_hook_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg,
 		 gpointer data)
 {
   static glong id = -1;
- 
+
   if (mix_vm_cmd_dispatcher_get_last_result (dis))
     {
       GtkWindow *mainw =
@@ -129,7 +145,7 @@ load_post_hook_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg,
       mixgtk_mixal_load_file ();
       mixgtk_mixal_update ();
       mixgtk_mixal_update_bp_all ();
-      
+
       if (id != -1)
 	gtk_statusbar_remove (GTK_STATUSBAR (dis_data_.status),
 			      dis_data_.context, (guint)id);
@@ -232,7 +248,7 @@ read_config_ (void)
     {
       mix_vm_cmd_dispatcher_set_editor (dis_data_.dispatcher, editor);
     }
-  if (!assem) assem = "mixasm -g %s";
+  if (!assem) assem = "mixasm %s";
   mix_vm_cmd_dispatcher_set_assembler (dis_data_.dispatcher, assem);
 
 }
@@ -257,23 +273,16 @@ on_external_programs_activate ()
 		      mix_vm_cmd_dispatcher_get_assembler
 		      (dis_data_.dispatcher));
   gtk_widget_show (ext_dlg_);
-}
+  if (gtk_dialog_run (GTK_DIALOG (ext_dlg_)) == GTK_RESPONSE_OK)
+    {
+      const gchar *value = gtk_entry_get_text (GTK_ENTRY (ed_entry_));
+      mix_vm_cmd_dispatcher_set_editor (dis_data_.dispatcher, value);
+      mixgtk_config_update (EDITOR_KEY_, value);
+      value = gtk_entry_get_text (GTK_ENTRY (asm_entry_));
+      mix_vm_cmd_dispatcher_set_assembler (dis_data_.dispatcher, value);
+      mixgtk_config_update (MIXASM_KEY_, value);
+    }
 
-void
-on_extern_cancel_button_clicked ()
-{
-  gtk_widget_hide (ext_dlg_);
-}
-
-void
-on_extern_ok_button_clicked ()
-{
-  const gchar *value = gtk_entry_get_text (GTK_ENTRY (ed_entry_));
-  mix_vm_cmd_dispatcher_set_editor (dis_data_.dispatcher, value);
-  mixgtk_config_update (EDITOR_KEY_, value);
-  value = gtk_entry_get_text (GTK_ENTRY (asm_entry_));
-  mix_vm_cmd_dispatcher_set_assembler (dis_data_.dispatcher, value);
-  mixgtk_config_update (MIXASM_KEY_, value);
   gtk_widget_hide (ext_dlg_);
 }
 
@@ -291,32 +300,33 @@ mixgtk_cmd_dispatcher_init (mixgtk_dialog_id_t top)
   dis_data_.prompt =
     mixgtk_widget_factory_get (top, MIXGTK_WIDGET_PROMPT);
   g_return_val_if_fail (dis_data_.prompt != NULL, FALSE);
-  
+
   if (dis_data_.log)
     text = gtk_editable_get_chars (GTK_EDITABLE (dis_data_.log), 0, -1);
-  
+
   dis_data_.log =
     mixgtk_widget_factory_get (top, MIXGTK_WIDGET_LOG);
-  
+
   g_return_val_if_fail (dis_data_.log != NULL, FALSE);
-  
+
   if (text)
     {
-      gtk_text_insert (GTK_TEXT (dis_data_.log), NULL, NULL, NULL, text, -1);
+      gtk_text_buffer_insert_at_cursor
+        (gtk_text_view_get_buffer (GTK_TEXT_VIEW (dis_data_.log)), text, -1);
       g_free (text);
     }
-  
+
   if (!dis_data_.dispatcher)
     {
       static const gchar *HISTORY_FILE = "gmixvm.history";
       static gint HISTORY_SIZE = 100;
       mix_config_t *config = mixgtk_config_get_mix_config ();
-      
+
       int r = pipe (dis_data_.fildes);
       g_return_val_if_fail (r == 0, FALSE);
       /* connect stdout/stderr to the pipe's write end */
       if (dup2 (dis_data_.fildes[1], STDOUT_FILENO) == -1
-	  || dup2 (dis_data_.fildes[1], STDERR_FILENO) == -1)
+	  || dup2 (dis_data_.fildes[1], STDOUT_FILENO) == -1)
 	return FALSE;
       dis_data_.out = fdopen (dis_data_.fildes[1], "w");
       g_return_val_if_fail (dis_data_.out != NULL, FALSE);
@@ -336,10 +346,10 @@ mixgtk_cmd_dispatcher_init (mixgtk_dialog_id_t top)
 					       dis_data_.out,
 					       config);
       mix_vm_cmd_dispatcher_print_time (dis_data_.dispatcher, FALSE);
-      
+
       install_hooks_ ();
     }
-  
+
   dis_data_.status =
     mixgtk_widget_factory_get (MIXGTK_MAIN, MIXGTK_WIDGET_STATUSBAR);
   g_return_val_if_fail (dis_data_.status != NULL, FALSE);
@@ -352,6 +362,9 @@ mixgtk_cmd_dispatcher_init (mixgtk_dialog_id_t top)
       (GTK_WINDOW (mixgtk_widget_factory_get_dialog (MIXGTK_MAIN)),
        dis_data_.last_file);
 
+  mixgtk_fontsel_set_font (MIX_FONT_LOG, dis_data_.log);
+  mixgtk_fontsel_set_font (MIX_FONT_PROMPT, dis_data_.prompt);
+
   restart = TRUE;
   return TRUE;
 }
@@ -424,38 +437,41 @@ complete_command_ (void)
 {
   GtkEntry *entry = GTK_ENTRY (dis_data_.prompt);
   gchar *prefix = NULL;
-  gchar *text = gtk_entry_get_text (entry);
+  const gchar *text = gtk_entry_get_text (entry);
   const GList  *cmds =
     mix_vm_cmd_dispatcher_complete (dis_data_.dispatcher, text, &prefix);
 
   if (prefix != NULL)
     {
+      GtkTextBuffer *buf =
+        gtk_text_view_get_buffer (GTK_TEXT_VIEW (dis_data_.log));
+
       gtk_entry_set_text (entry, prefix);
-      g_free (prefix);
       if (g_list_length ((GList *)cmds) > 1)
 	{
-	  static gchar BUFFER[25];
+          static gchar BUFFER[25];
 	  gint k = 0;
-	  gtk_text_insert (GTK_TEXT (dis_data_.log),
-			   NULL, NULL, NULL, "Completions:\n", -1);
+	  gtk_text_buffer_insert_at_cursor (buf, "Completions:\n", -1);
 	  while (cmds)
 	    {
 	      g_snprintf (BUFFER, 25, "%-12s", (const char*)cmds->data);
 	      ++k;
-	      gtk_text_insert (GTK_TEXT (dis_data_.log),
-			       NULL, NULL, NULL, BUFFER, -1);
+	      gtk_text_buffer_insert_at_cursor (buf, BUFFER, -1);
 	      if (k%5 == 0)
-		gtk_text_insert (GTK_TEXT (dis_data_.log),
-				 NULL, NULL, NULL, "\n", -1);
-	      
+		gtk_text_buffer_insert_at_cursor (buf, "\n", -1);
+
 	      cmds = cmds->next;
 	    }
 	  if (k%5 != 0)
-	    gtk_text_insert (GTK_TEXT (dis_data_.log),
-			     NULL, NULL, NULL, "\n", -1);
+	    gtk_text_buffer_insert_at_cursor (buf, "\n", -1);
 	}
       else
-	gtk_entry_append_text (entry, " ");
+        {
+          gint pos = strlen (prefix);
+          gtk_editable_insert_text (GTK_EDITABLE (entry), " ", 1, &pos);
+        }
+      flush_log_ (&dis_data_);
+      g_free (prefix);
     }
 }
 
@@ -463,6 +479,7 @@ int
 on_command_prompt_key_press_event (GtkEntry *w, GdkEventKey *e, gpointer d)
 {
   guint key = e->keyval;
+  gboolean result = FALSE;
 
 #ifdef HAVE_LIBHISTORY
   HIST_ENTRY *entry = NULL;
@@ -471,35 +488,60 @@ on_command_prompt_key_press_event (GtkEntry *w, GdkEventKey *e, gpointer d)
       entry = previous_history ();
       if (entry && entry->line)
 	gtk_entry_set_text (w, entry->line);
-      return TRUE;
+      result = TRUE;
     }
   if (key == GDK_Down)
     {
       entry = next_history ();
       if (entry && entry->line)
 	gtk_entry_set_text (w, entry->line);
-      return TRUE;
+      result = TRUE;
     }
 #endif
 
   if (key == GDK_Tab)
     {
       complete_command_ ();
-      return TRUE;
+      result = TRUE;
     }
 
-  if (key == GDK_Return)
+  if (result) gtk_editable_set_position (GTK_EDITABLE (w), -1);
+
+  return result;
+}
+
+void
+on_command_prompt_activate (GtkEntry *prompt, gpointer data)
+{
+  gchar *text =
+    g_strstrip (gtk_editable_get_chars (GTK_EDITABLE (prompt), 0, -1));
+  if (text && *text)
     {
-      gchar *text = g_strstrip (gtk_entry_get_text (w));
-      if (text && *text)
-	{
-	  log_command_ (&dis_data_, text);
-	  if (!try_guile_ (text))
-	    mix_vm_cmd_dispatcher_dispatch_text (dis_data_.dispatcher, text);
-	  gtk_entry_set_text (w, "");
-	}
-      return TRUE;
+      log_command_ (&dis_data_, text);
+      if (!try_guile_ (text))
+        mix_vm_cmd_dispatcher_dispatch_text (dis_data_.dispatcher, text);
+      gtk_entry_set_text (prompt, "");
     }
-  
-  return FALSE;
+  g_free (text);
+}
+
+void
+on_log_font_activate ()
+{
+  GtkWidget *w[] = { dis_data_.log };
+  mixgtk_fontsel_query_font (MIX_FONT_LOG, w, 1);
+}
+
+void
+on_prompt_font_activate ()
+{
+  GtkWidget *w[] = { dis_data_.prompt };
+  mixgtk_fontsel_query_font (MIX_FONT_PROMPT, w, 1);
+}
+
+void
+mixgtk_cmd_dispatcher_update_fonts (void)
+{
+  mixgtk_fontsel_set_font (MIX_FONT_LOG, dis_data_.log);
+  mixgtk_fontsel_set_font (MIX_FONT_PROMPT, dis_data_.prompt);
 }
