@@ -1,24 +1,24 @@
 /* -*-c-*- -------------- mixgtk_config.c :
  * Implementation of the functions declared in mixgtk_config.h
  * ------------------------------------------------------------------
- *  $Id: mixgtk_config.c,v 1.8 2002/03/29 16:30:49 jao Exp $
+ *  $Id: mixgtk_config.c,v 1.9 2004/06/30 14:07:53 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2001, 2002 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2001, 2002, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 #include <stdlib.h>
@@ -32,9 +32,6 @@
 #include "mixgtk_config.h"
 
 static const gchar *MIXGTK_CONFIG_FILE_ = "gmixvm.config";
-static const gchar *SPLIT_KEY = "Split";
-static const gchar *SPLIT_YES = "Yes";
-static const gchar *SPLIT_NO = "No";
 static const gchar *SHOW_TB_KEY = "Toolbars";
 static const gchar *TB_YES = "Yes";
 static const gchar *TB_NO = "No";
@@ -103,20 +100,7 @@ mixgtk_config_save (void)
 }
 
 gboolean
-mixgtk_config_is_split (void)
-{
-  const gchar *split = mixgtk_config_get (SPLIT_KEY);
-  return (split && !strcmp (split, SPLIT_YES));
-}
-
-void
-mixgtk_config_set_split (gboolean split)
-{
-  mixgtk_config_update (SPLIT_KEY, split? SPLIT_YES : SPLIT_NO);
-}
-
-gboolean
-mixgtk_config_show_toolbars (void) 
+mixgtk_config_show_toolbars (void)
 {
   const gchar *show = mixgtk_config_get (SHOW_TB_KEY);
   return (!show || !strcmp (show, TB_YES));
