@@ -1,24 +1,24 @@
 /* -*-c-*- ---------------- mixgtk_config.h :
  * Configuration functions declarations.
  * ------------------------------------------------------------------
- *  $Id: mixgtk_config.h,v 1.5 2001/09/18 22:50:30 jao Exp $
+ *  $Id: mixgtk_config.h,v 1.6 2004/06/30 14:07:53 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2001 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 
@@ -60,12 +60,6 @@ extern void
 mixgtk_config_save (void);
 
 /* shared config params */
-extern gboolean
-mixgtk_config_is_split (void);
-
-extern void
-mixgtk_config_set_split (gboolean split);
-
 extern gboolean
 mixgtk_config_show_toolbars (void);
 
