@@ -1,7 +1,7 @@
 /* -*-c-*- ---------------- mixgtk_device.c :
  * actual types for mixgtk devices
  * ------------------------------------------------------------------
- * $Id: mixgtk_device.c,v 1.20 2004/06/11 23:35:59 jao Exp $
+ * $Id: mixgtk_device.c,v 1.25 2004/07/04 22:34:15 jao Exp $
  * ------------------------------------------------------------------
  * Copyright (C) 2001, 2002, 2004 Free Software Foundation, Inc.
  *
@@ -32,6 +32,7 @@
 #include "mixgtk.h"
 #include "mixgtk_gen_handlers.h"
 #include "mixgtk_widgets.h"
+#include "mixgtk_fontsel.h"
 #include "mixgtk_config.h"
 #include "mixgtk_cmd_dispatcher.h"
 #include "mixgtk_device.h"
@@ -41,7 +42,7 @@
 static const gchar *DEV_FORMAT_KEY_ = "Device.format";
 
 /* device container */
-static GtkNotebook *container_ = NULL;
+static GtkNotebook *dev_nb_ = NULL;
 /* devdir dialog */
 static GtkWidget *devdir_dlg_ = NULL;
 static GtkEntry *devdir_entry_ = NULL;
@@ -56,7 +57,6 @@ static GtkWidget *dtoggle_ = NULL;
 static GtkWidget *wtoggle_ = NULL;
 static GtkWidget *combo_ = NULL;
 
-static gint last_pos_ = 0;
 /* virtual machine */
 static mix_vm_t *vm_ = NULL;
 /* dec settings */
@@ -73,14 +73,14 @@ struct mixgtk_device_t
 {
   mix_device_t device;
   GtkWidget *widget;
-  gint pos;
+  GtkWidget *scroll;
 };
 
 struct mixgtk_bin_device_t
 {
   struct mixgtk_device_t gtk_device;
   guint last_insert;
-  GtkWidget *scroll;
+  GtkListStore *store;
   gboolean dec;
 };
 
@@ -90,7 +90,11 @@ write_char_ (struct mixgtk_device_t *dev, const mix_word_t *block)
 {
   enum {MAX_BLOCK = 16, BUFF_SIZE = MAX_BLOCK * 5 + 2};
   static gchar BUFFER[BUFF_SIZE];
+
   guint k, j;
+  GtkTextIter end;
+  GtkTextBuffer *buffer;
+  GtkTextMark *mark;
 
   for (k = 0; k < SIZES_[dev->device.type]; k++)
     for (j = 1; j < 6; j++)
@@ -102,8 +106,13 @@ write_char_ (struct mixgtk_device_t *dev, const mix_word_t *block)
   BUFFER[5 * k] = '\n';
   BUFFER[5 * k + 1] = '\0';
 
-  gtk_text_insert (GTK_TEXT (dev->widget), NULL, NULL, NULL, BUFFER, -1);
-
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dev->widget));
+  gtk_text_buffer_get_end_iter (buffer, &end);
+  gtk_text_buffer_place_cursor (buffer, &end);
+  gtk_text_buffer_insert_at_cursor (buffer, BUFFER, -1);
+  mark = gtk_text_buffer_get_insert (buffer);
+  gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (dev->widget), mark, 0,
+                                TRUE, 0, 0);
 }
 
 static const gchar *
@@ -123,64 +132,32 @@ get_word_string_ (mix_word_t w, gboolean dec)
 static void
 write_bin_  (struct mixgtk_bin_device_t *dev, const mix_word_t *block)
 {
-  static gchar *DEFTEXT[BIN_DEV_COL_] = { "0", "0", "0", "0", "0" };
-
-  guint k, col, row;
+  guint k, col;
   gboolean dec = FALSE;
+  size_t len;
 
-  GtkCList *list = GTK_CLIST (dev->gtk_device.widget);
+  GtkTreeView *view = GTK_TREE_VIEW (dev->gtk_device.widget);
+  GtkListStore *store = dev->store;
+  GtkTreeIter iter;
+
+  g_assert (view);
+  g_assert (store);
 
   dec = IS_DEC(decs_, dev->gtk_device.device.type);
 
-  gtk_clist_freeze (list);
-
-  for (k = 0; k < SIZES_[dev->gtk_device.device.type]; k++)
+  for (k = 0, len = SIZES_[dev->gtk_device.device.type];
+       k < len;  k += BIN_DEV_COL_)
     {
-      row = dev->last_insert / BIN_DEV_COL_;
-      col = dev->last_insert % BIN_DEV_COL_;
-      if (col == 0)
-	{
-	  int j;
-	  mix_word_t *words = g_new (mix_word_t, 5);
-	  gtk_clist_append (list, DEFTEXT);
-	  for (j = 0; j < 5; ++j) words[j] = block[k + j];
-	  gtk_clist_set_row_data_full (list, row, (gpointer)words, g_free);
-	}
-      gtk_clist_set_text (list, row, col, get_word_string_ (block[k], dec));
+      gtk_list_store_append (store, &iter);
+      for (col = 0; col < BIN_DEV_COL_; ++col)
+        gtk_list_store_set (store, &iter,
+                            col, get_word_string_ (block[k + col], dec),
+                            col + BIN_DEV_COL_, (guint)block[k + col],
+                            -1);
       dev->last_insert++;
     }
-
-  gtk_clist_thaw (list);
 }
 
-static void
-redraw_bin_device_ (struct mixgtk_bin_device_t *dev)
-{
-  gint k, col, row;
-  gboolean dec = FALSE;
-  mix_word_t *words = NULL;
-  GtkCList *list;
-
-  if (!dev) return;
-
-  list = GTK_CLIST (dev->gtk_device.widget);
-  dec = IS_DEC(decs_, dev->gtk_device.device.type);
-
-  gtk_clist_freeze (list);
-
-  for (k = 0; k < dev->last_insert; ++k)
-    {
-      row = k / BIN_DEV_COL_;
-      col = k % BIN_DEV_COL_;
-      if (col == 0)
-	words = gtk_clist_get_row_data (list, row);
-      gtk_clist_set_text (list, row, col, get_word_string_ (words[col], dec));
-    }
-
-  gtk_clist_thaw (list);
-}
-
-
 static gboolean
 write_ (mix_device_t *dev, const mix_word_t *block)
 {
@@ -192,7 +169,9 @@ write_ (mix_device_t *dev, const mix_word_t *block)
   if (MODES_[dev->type] == mix_dev_CHAR) write_char_ (gtkdev, block);
   else write_bin_ ((struct mixgtk_bin_device_t *)gtkdev, block);
 
-  gtk_notebook_set_page (container_, gtkdev->pos);
+  gtk_notebook_set_current_page (dev_nb_,
+                                 gtk_notebook_page_num (dev_nb_,
+                                                        gtkdev->scroll));
 
   return TRUE;
 }
@@ -212,8 +191,9 @@ read_cons_ (mix_word_t *block)
       g_assert (input_dlg_entry_);
     }
 
-  gtk_widget_show (input_dlg_);
-  gtk_main ();              /* wait until dialog closes */
+  gtk_entry_set_text (input_dlg_entry_, "");
+  gtk_dialog_run (GTK_DIALOG (input_dlg_));
+  gtk_widget_hide (input_dlg_);
   text = g_strdup_printf ("%-70s", gtk_entry_get_text (input_dlg_entry_));
   for (i = 0; i < 70; ++i)
     for (j = 0; j < 5; ++j)
@@ -237,8 +217,9 @@ read_ (mix_device_t *dev, mix_word_t *block)
   if (MODES_[dev->type] == mix_dev_CHAR) write_char_ (gtkdev, block);
   else write_bin_ ((struct mixgtk_bin_device_t *)gtkdev, block);
 
-  gtk_notebook_set_page (container_, gtkdev->pos);
-
+  gtk_notebook_set_current_page (dev_nb_,
+                                 gtk_notebook_page_num (dev_nb_,
+                                                        gtkdev->scroll));
   return TRUE;
 }
 
@@ -258,12 +239,15 @@ static void
 destroy_ (mix_device_t *dev)
 {
   struct mixgtk_device_t *gtkdev = (struct mixgtk_device_t *)dev;
-  if (MODES_[dev->type] == mix_dev_BIN)
-    gtk_widget_destroy (((struct mixgtk_bin_device_t *)dev)->scroll);
-  gtk_widget_destroy (gtkdev->widget);
+  if (MODES_[dev->type] == mix_dev_BIN) {
+    struct mixgtk_bin_device_t * bdev = (struct mixgtk_bin_device_t *)dev;
+    gtk_list_store_clear (bdev->store);
+    g_object_unref ((gpointer)bdev->store);
+  }
   (DEF_DEV_VTABLE_->destroy) (dev);
-  gtk_notebook_remove_page (GTK_NOTEBOOK (container_), gtkdev->pos);
-  --last_pos_;
+  gtk_notebook_remove_page (dev_nb_,
+                            gtk_notebook_page_num (dev_nb_,
+                                                   gtkdev->scroll));
 }
 
 static mix_device_vtable_t MIXGTK_VTABLE_ = {
@@ -278,54 +262,92 @@ mixgtk_device_construct_gui_ (struct mixgtk_device_t *dev)
 
   g_assert (label);
 
-  dev->pos = last_pos_++;
+  dev->scroll = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dev->scroll),
+                                  GTK_POLICY_AUTOMATIC,
+                                  GTK_POLICY_AUTOMATIC);
+
   if (MODES_[dev->device.type] == mix_dev_CHAR)
     {
-      GtkWidget *box = gtk_hbox_new (0, 0);
-      GtkWidget *scroll = NULL;
-      GtkAdjustment *vadj = NULL;
-      dev->widget = gtk_text_new (NULL, NULL);
-      g_assert (box);
-      g_assert (dev->widget);
-      gtk_text_set_editable (GTK_TEXT (dev->widget), FALSE);
-      vadj = GTK_TEXT (dev->widget)->vadj;
-      scroll = gtk_vscrollbar_new (vadj);
-      gtk_box_pack_start (GTK_BOX (box), dev->widget, TRUE, TRUE, 0);
-      gtk_box_pack_start (GTK_BOX (box), scroll, FALSE, FALSE, 0);
-      gtk_notebook_insert_page (container_, box, label, dev->pos);
-      gtk_widget_show (box);
-      gtk_widget_show (scroll);
+      dev->widget = gtk_text_view_new ();
+      gtk_text_view_set_editable (GTK_TEXT_VIEW (dev->widget), FALSE);
     }
   else
     {
       gint k;
-      struct mixgtk_bin_device_t *bindev =
-	(struct mixgtk_bin_device_t *)dev;
+      struct mixgtk_bin_device_t *bindev = (struct mixgtk_bin_device_t *)dev;
+      GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
+      GType *types = g_new (GType, BIN_DEV_COL_ * 2);
 
-      bindev->scroll = gtk_scrolled_window_new (NULL, NULL);
-      g_assert (bindev->scroll);
-      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (bindev->scroll),
-				      GTK_POLICY_AUTOMATIC,
-				      GTK_POLICY_AUTOMATIC);
-      dev->widget = gtk_clist_new (BIN_DEV_COL_);
-      g_assert (dev->widget);
-      for (k =0; k < BIN_DEV_COL_; ++k)
+      for (k = 0; k < BIN_DEV_COL_; ++k)
+        {
+          types[k] = G_TYPE_STRING;
+          types[k + BIN_DEV_COL_] = G_TYPE_UINT;
+        }
+
+      bindev->store = gtk_list_store_newv (BIN_DEV_COL_ * 2, types);
+
+      dev->widget =
+        gtk_tree_view_new_with_model (GTK_TREE_MODEL (bindev->store));
+
+      gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dev->widget), FALSE);
+      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (dev->widget), TRUE);
+
+      for (k = 0; k < BIN_DEV_COL_; ++k)
 	{
-	  gtk_clist_set_column_width (GTK_CLIST (dev->widget), k, 120);
-	  gtk_clist_set_column_auto_resize (GTK_CLIST (dev->widget), k, TRUE);
+          gtk_tree_view_append_column
+            (GTK_TREE_VIEW (dev->widget),
+             gtk_tree_view_column_new_with_attributes ("", renderer,
+                                                       "text", k, NULL));
 	}
-      gtk_container_add (GTK_CONTAINER (bindev->scroll), dev->widget);
-      gtk_notebook_insert_page (container_, bindev->scroll, label, dev->pos);
-      gtk_widget_show (bindev->scroll);
+
+
+      g_free (types);
     }
 
+  g_assert (dev->widget);
+  gtk_container_add (GTK_CONTAINER (dev->scroll), dev->widget);
+  mixgtk_fontsel_set_font (MIX_FONT_DEVICES, dev->widget);
+  gtk_notebook_append_page (dev_nb_, dev->scroll, label);
   gtk_widget_show (label);
-  gtk_widget_set_style (dev->widget,
-			gtk_widget_get_style (GTK_WIDGET (container_)));
+  gtk_widget_show (dev->scroll);
   gtk_widget_show (dev->widget);
-  gtk_widget_draw (GTK_WIDGET (container_), NULL);
 }
 
+static void
+redraw_bin_device_ (struct mixgtk_bin_device_t *dev)
+{
+  if (dev != NULL)
+    {
+      GtkTreeModel *store = GTK_TREE_MODEL (dev->store);
+      GtkTreeIter iter;
+      gboolean valid;
+
+      g_assert (store);
+
+      valid =
+        gtk_tree_model_get_iter_first (store, &iter);
+
+      while (valid)
+        {
+          gint k;
+          guint word;
+
+          for (k = 0; k < BIN_DEV_COL_; ++k)
+            {
+              gtk_tree_model_get (store, &iter, k + BIN_DEV_COL_, &word, -1);
+              gtk_list_store_set (GTK_LIST_STORE (store), &iter,
+                                  k, get_word_string_ ((mix_word_t)word,
+                                                       dev->dec),
+                                  -1);
+            }
+          valid = gtk_tree_model_iter_next (store, &iter);
+        }
+      gtk_widget_queue_draw (((struct mixgtk_device_t *)dev)->widget);
+    }
+}
+
+
 /* create a new mixgtk device */
 static mix_device_t *
 mixgtk_device_new_ (mix_device_type_t type)
@@ -353,57 +375,13 @@ mixgtk_device_new_ (mix_device_type_t type)
   return (mix_device_t *)dev;
 }
 
-static mix_device_t *
-mixgtk_device_copy_ (const struct mixgtk_device_t *from)
-{
-  struct mixgtk_device_t *result =
-    (struct mixgtk_device_t *)mixgtk_device_new_ (from->device.type);
-  g_return_val_if_fail (result != NULL, NULL);
-  if (MODES_[from->device.type] == mix_dev_CHAR)
-    {
-      gchar *text = gtk_editable_get_chars (GTK_EDITABLE (from->widget),
-					    0, -1);
-      gtk_text_insert (GTK_TEXT (result->widget), NULL, NULL, NULL, text, -1);
-      g_free (text);
-    }
-  else
-    {
-      static gchar *VALS[BIN_DEV_COL_] = {""};
-      struct mixgtk_bin_device_t *to = (struct mixgtk_bin_device_t *) result;
-      struct mixgtk_bin_device_t *fr = (struct mixgtk_bin_device_t *) from;
-      to->last_insert = fr->last_insert;
-      to->dec = fr->dec;
-      if (to->last_insert > 0)
-	{
-	  gint k, j, rows =  to->last_insert / BIN_DEV_COL_;
-	  GtkCList *tl = GTK_CLIST (result->widget);
-	  GtkCList *fl = GTK_CLIST (from->widget);
-	  for (k = 0; k < rows; ++k)
-	    {
-	      mix_word_t *words = g_new (mix_word_t, BIN_DEV_COL_);
-	      mix_word_t *oldwords =
-		(mix_word_t *) gtk_clist_get_row_data (fl, k);
-	      for (j = 0; j < BIN_DEV_COL_; ++j) words[j] = oldwords[j];
-	      gtk_clist_append (tl, VALS);
-	      gtk_clist_set_row_data_full (tl, k, words, g_free);
-	    }
-	  redraw_bin_device_ (to);
-	}
-    }
-  return (mix_device_t *)result;
-}
-
 /* init default devices */
-gboolean
-mixgtk_device_init (GtkNotebook *container, mix_vm_t *vm)
+GtkWidget *
+mixgtk_device_init (mix_vm_t *vm)
 {
-  gint k = 0;
+  g_assert (vm != NULL);
 
-  g_return_val_if_fail (container != NULL, FALSE);
-  g_return_val_if_fail (vm != NULL, FALSE);
-  container_ = container;
   vm_ = vm;
-  last_pos_ = 0;
   devdlg_ = NULL;
   devdir_dlg_ = NULL;
   devdir_entry_ = NULL;
@@ -412,31 +390,22 @@ mixgtk_device_init (GtkNotebook *container, mix_vm_t *vm)
   wtoggle_ = NULL;
   combo_ = NULL;
 
-  /* remove dummy page from container */
-  gtk_notebook_remove_page (container_, 0);
-
-  mix_vm_set_device_factory (vm, mixgtk_device_new_);
-
-  /* read format configuration */
-  if (mixgtk_config_get (DEV_FORMAT_KEY_))
-    decs_ =  atoi (mixgtk_config_get (DEV_FORMAT_KEY_));
-
-  /* re-create existing devices */
-  for (k = 0; k < mix_dev_INVALID; ++k)
+  if (dev_nb_ == NULL)
     {
-      mix_device_t *dev = mix_vm_get_device (vm, k);
-      if (dev != NULL)
-	{
-	  mix_device_t *newdev =
-	    mixgtk_device_copy_ ((struct mixgtk_device_t *)dev);
-	  mix_vm_connect_device (vm, newdev);
-	}
+      mix_vm_set_device_factory (vm, mixgtk_device_new_);
+
+      /* read format configuration */
+      if (mixgtk_config_get (DEV_FORMAT_KEY_))
+        decs_ =  atoi (mixgtk_config_get (DEV_FORMAT_KEY_));
+
+      dev_nb_ = GTK_NOTEBOOK (gtk_notebook_new ());
+      gtk_notebook_set_tab_pos (dev_nb_, GTK_POS_RIGHT);
+      gtk_notebook_set_scrollable (dev_nb_, FALSE);
     }
 
-  /* set to first page */
-  gtk_notebook_set_page (container_, 0);
+  mixgtk_fontsel_set_font (MIX_FONT_DEVICES, GTK_WIDGET (dev_nb_));
 
-  return TRUE;
+  return GTK_WIDGET (dev_nb_);
 }
 
 void
@@ -450,34 +419,32 @@ mixgtk_device_set_format (mix_device_type_t dev, gboolean dec)
 
   if (changed && (MODES_[dev] == mix_dev_BIN))
     {
-      if (dec) SET_DEC (decs_, dev);
-      else CLEAR_DEC (decs_, dev);
-      redraw_bin_device_ ((struct mixgtk_bin_device_t *)
-			  mix_vm_get_device (vm_, dev));
-    }
+      struct mixgtk_bin_device_t *gdev =
+        (struct mixgtk_bin_device_t *) mix_vm_get_device (vm_, dev);
 
+      if (gdev != NULL)
+        {
+          if (dec) SET_DEC (decs_, dev); else CLEAR_DEC (decs_, dev);
+          gdev->dec = !(gdev->dec);
+          redraw_bin_device_ (gdev);
+        }
+    }
 }
 
 
 static mix_device_type_t
 get_device_idx_ (void)
 {
-  const gchar *name =  gtk_entry_get_text
-    (GTK_ENTRY (GTK_COMBO (combo_)->entry));
-  /* inefficient, but the list is short */
   int k;
+  const gchar *name = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (combo_)->child));
   for (k = 0; k <= LAST_BIN_DEV_; ++k)
-    if (!strcmp (name, DEF_NAMES_[k])) return k;
-  g_assert_not_reached ();
-  return 0;
+    if (!strcmp (name, DEF_NAMES_[k])) break;
+  return k;
 }
 
 static void
 init_devform_ (void)
 {
-  GList *names = NULL;
-  int k;
-
   devdlg_ = mixgtk_widget_factory_get_dialog (MIXGTK_DEVFORM_DIALOG);
   g_assert (devdlg_);
   dtoggle_ = mixgtk_widget_factory_get_child_by_name
@@ -489,29 +456,47 @@ init_devform_ (void)
   combo_ = mixgtk_widget_factory_get_child_by_name
     (MIXGTK_DEVFORM_DIALOG, "dev_combo");
   g_assert (combo_);
-
-  for (k = 0; k <= LAST_BIN_DEV_; ++k)
-    names = g_list_append (names, (gchar *)DEF_NAMES_[k]);
-  gtk_combo_set_popdown_strings (GTK_COMBO (combo_), names);
+  gtk_editable_set_editable (GTK_EDITABLE (GTK_BIN (combo_)->child), FALSE);
 }
 
 void
-on_deventry_changed ()
+on_dev_combo_changed ()
 {
   mix_device_type_t dev = get_device_idx_ ();
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dtoggle_),
-				IS_DEC (new_decs_, dev));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wtoggle_),
-				!IS_DEC (new_decs_, dev));
+  if (dev <= LAST_BIN_DEV_)
+    {
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dtoggle_),
+                                    IS_DEC (new_decs_, dev));
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wtoggle_),
+                                    !IS_DEC (new_decs_, dev));
+    }
 }
 
 void
 on_devform_activate ()
 {
+  gint result = GTK_RESPONSE_APPLY;
+
   if (!devdlg_) init_devform_ ();
   new_decs_ = decs_;
-  gtk_widget_show (devdlg_);
-  on_deventry_changed ();
+
+  while (result == GTK_RESPONSE_APPLY)
+    {
+      //on_dev_combo_changed ();
+      result = gtk_dialog_run (GTK_DIALOG (devdlg_));
+      if (result != GTK_RESPONSE_CANCEL)
+        {
+          int k;
+          gchar value[20];
+          for (k = 0; k <= LAST_BIN_DEV_; ++k)
+            mixgtk_device_set_format (k, IS_DEC (new_decs_, k));
+          decs_ = new_decs_;
+          g_snprintf (value, 20, "%d", decs_);
+          mixgtk_config_update (DEV_FORMAT_KEY_, value);
+        }
+    }
+
+  gtk_widget_hide (devdlg_);
 }
 
 void
@@ -523,31 +508,6 @@ on_decradio_toggled (GtkToggleButton *button)
     CLEAR_DEC (new_decs_, get_device_idx_ ());
 }
 
-void
-on_devapp_button_clicked ()
-{
-  int k;
-  gchar value[20];
-  for (k = 0; k <= LAST_BIN_DEV_; ++k)
-    mixgtk_device_set_format (k, IS_DEC (new_decs_, k));
-  decs_ = new_decs_;
-  g_snprintf (value, 20, "%d", decs_);
-  mixgtk_config_update (DEV_FORMAT_KEY_, value);
-}
-
-void
-on_devok_button_clicked ()
-{
-  on_devapp_button_clicked ();
-  gtk_widget_hide (devdlg_);
-}
-
-void
-on_devcancel_button_clicked ()
-{
-  gtk_widget_hide (devdlg_);
-}
-
 void
 on_devset_button_clicked ()
 {
@@ -573,16 +533,32 @@ on_devdir_activate ()
   gtk_widget_show (devdir_dlg_);
 }
 
-static void
-devdir_callback (const gchar *file)
-{
-  gtk_entry_set_text (devdir_entry_, file);
-}
-
 void
 on_devdir_browse_clicked ()
 {
-  mixgtk_get_file (devdir_callback, _("Devices dir"), mix_device_get_dir ());
+  GtkWidget *dialog;
+  const gchar *current = gtk_entry_get_text (devdir_entry_);
+
+  dialog =
+    gtk_file_chooser_dialog_new (_("Devices folder"),
+                                 GTK_WINDOW (devdir_dlg_),
+                                 GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                 NULL);
+
+  if (current != NULL)
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), current);
+
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+    {
+      char *filename
+        = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+      gtk_entry_set_text (devdir_entry_, filename);
+      g_free (filename);
+    }
+
+  gtk_widget_destroy (dialog);
 }
 
 void
@@ -602,21 +578,48 @@ on_devdir_ok_clicked ()
   g_free (cmd);
 }
 
-void
-on_input_ok_clicked ()
+
+
+/* fonts */
+static GtkWidget **
+font_widgets_ (gint * n)
 {
-  gtk_widget_hide (input_dlg_);
-  gtk_main_quit ();
+  gint no = gtk_notebook_get_n_pages (dev_nb_);
+  GtkWidget **w = g_new (GtkWidget *, no + 1);
+  gint k, j;
+
+  for (k = 0, j = 0; k < no && j < mix_dev_INVALID; ++j)
+    {
+      struct mixgtk_device_t * dev =
+        (struct mixgtk_device_t *) mix_vm_get_device (vm_, j);
+      if (dev != NULL) w[k++] = dev->widget;
+    }
+
+  g_assert (k == no);
+  w[k] = GTK_WIDGET (dev_nb_);
+
+  if (n) *n = no;
+
+  return w;
 }
 
 void
-on_input_clear_clicked ()
+on_devices_font_activate ()
 {
-  gtk_entry_set_text (input_dlg_entry_, "");
+  gint no = 0;
+  GtkWidget **w = font_widgets_ (&no);
+  mixgtk_fontsel_query_font (MIX_FONT_DEVICES, w, no + 1);
+  g_free (w);
 }
 
 void
-on_input_entry_activate ()
+mixgtk_device_update_fonts (void)
 {
-  on_input_ok_clicked ();
+  gint k, n = 0;
+  GtkWidget **w = font_widgets_ (&n);
+
+  g_assert (n >= 0);
+
+  for (k = 0; k < n; ++k)
+    mixgtk_fontsel_set_font (MIX_FONT_DEVICES, w[k]);
 }
