@@ -3,22 +3,22 @@
  * ------------------------------------------------------------------
  *  Last change: Time-stamp: <01/03/04 23:43:59 jose>
  * ------------------------------------------------------------------
- * Copyright (C) 2001 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 
@@ -30,8 +30,11 @@
 #include <mixlib/mix_device.h>
 
 /* init default devices */
-extern gboolean
-mixgtk_device_init (GtkNotebook *container, mix_vm_t *vm);
+extern GtkWidget *
+mixgtk_device_init (mix_vm_t *vm);
+
+extern void
+mixgtk_device_update_fonts (void);
 
 /* change the output format of a device */
 extern void
