@@ -1,24 +1,24 @@
 /* -*-c-*- ---------------- mixgtk_fontsel.h :
  * Functions for handling font selection.
  * ------------------------------------------------------------------
- *  Last change: Time-stamp: <2001-04-28 00:42:39 jao>
+ *  $Id: mixgtk_fontsel.h,v 1.7 2004/07/05 21:27:56 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2001 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 
@@ -28,17 +28,23 @@
 #include <glib.h>
 #include "mixgtk_widgets.h"
 
-/* load default (configured) fonts */
-extern void
-mixgtk_fontsel_load_defaults (void);
+typedef enum {
+  MIX_FONT_MIXVM,
+  MIX_FONT_LOG,
+  MIX_FONT_PROMPT,
+  MIX_FONT_MIXAL,
+  MIX_FONT_DEVICES,
+  MIX_FONT_SYMBOLS,
+  MIX_FONT_DEFAULT,
+  MIX_FONT_NO
+} mixgtk_font_t;
 
-/* set a widget's font */
+/* configured fonts */
 extern void
-mixgtk_fontsel_set (mixgtk_widget_id_t widget, const gchar *font);
+mixgtk_fontsel_set_font (mixgtk_font_t f, GtkWidget *w);
 
-/* get a widget's font */
-extern const gchar *
-mixgtk_fontsel_get (mixgtk_widget_id_t widget);
+extern gboolean
+mixgtk_fontsel_query_font (mixgtk_font_t f, GtkWidget **ws, size_t no);
 
 
 #endif /* MIXGTK_FONTSEL_H */
