@@ -1,24 +1,24 @@
 /* -*-c-*- ---------------- mixgtk_gen_handlers.h :
  * general signal handlers declarations
  * ------------------------------------------------------------------
- *  Last change: Time-stamp: <2001-04-29 12:32:32 jao>
+ *  $Id: mixgtk_gen_handlers.h,v 1.6 2004/08/01 21:43:56 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2001 Free Software Foundation, Inc.
- *  
+ * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *  
+ *
  */
 
 
@@ -55,7 +55,7 @@ typedef void (*file_callback_t)(const gchar *file);
 
 extern void
 mixgtk_get_file (file_callback_t callback, const gchar *title,
-		 const gchar *pattern);
+		 const gchar *pattern, const gchar *def_file);
 
 
 #endif /* MIXGTK_GEN_HANDLERS_H */
