@@ -1,7 +1,7 @@
 /* -*-c-*- ---------------- mixgtk_widgets.h :
  * glade-based factory of mixvm widgets
  * ------------------------------------------------------------------
- *  Last change: Time-stamp: <2001-04-27 23:32:16 jao>
+ * $Id: mixgtk_widgets.h,v 1.14 2004/06/30 22:46:15 jao Exp $
  * ------------------------------------------------------------------
  * Copyright (C) 2001, 2004 Free Software Foundation, Inc.
  *
@@ -37,23 +37,26 @@ typedef enum {
   MIXGTK_WORD_DIALOG,	  /* input dialog for a mix word */
   MIXGTK_ABOUT_DIALOG,
   MIXGTK_GOTO_DIALOG,
-  MIXGTK_COLOR_DIALOG,    /* color customization dialog */
-  MIXGTK_COLORSEL_DIALOG, /* color selection dialog */
   MIXGTK_FONTSEL_DIALOG,  /* font selection dialog */
   MIXGTK_DEVFORM_DIALOG,  /* device format config dialog */
   MIXGTK_EXTERNPROG_DIALOG,	/* external programs dialog */
   MIXGTK_SYMBOLS_DIALOG,	/* symbol table dialog */
   MIXGTK_INPUT_DIALOG,          /* console input dialog */
-  MIXGTK_DEVDIR_DIALOG
+  MIXGTK_DEVDIR_DIALOG          /* devices directory chooser */
 } mixgtk_dialog_id_t;
 
 /* enumeration of mixvm widget ids */
 typedef enum {
   MIXGTK_WIDGET_STATUSBAR,
+  MIXGTK_WIDGET_ATTACH_BUTTON,
+  MIXGTK_WIDGET_NOTEBOOK,       /* the notebook */
   MIXGTK_WIDGET_MIXVM,		/* virtual machine */
+  MIXGTK_WIDGET_MIXVM_CONTAINER,
   MIXGTK_WIDGET_PROMPT,		/* command prompt */
   MIXGTK_WIDGET_LOG,		/* command output */
-  MIXGTK_WIDGET_MIXAL,		/* mixal source file */
+  MIXGTK_WIDGET_DEV_CONTAINER,
+  MIXGTK_WIDGET_MIXAL_CONTAINER,
+  MIXGTK_WIDGET_MIXAL_STATUSBAR,
   MIXGTK_WIDGET_rA,
   MIXGTK_WIDGET_rX,
   MIXGTK_WIDGET_rJ,
@@ -71,9 +74,9 @@ typedef enum {
   MIXGTK_WIDGET_LAPTIME,
   MIXGTK_WIDGET_PROGTIME,
   MIXGTK_WIDGET_UPTIME,
-  MIXGTK_WIDGET_CELLS_SCROLL,
-  MIXGTK_WIDGET_DEVICE,
+  MIXGTK_WIDGET_DEVICES,
   MIXGTK_WIDGET_LOC,
+  MIXGTK_WIDGET_GOTO_ENTRY,
   MIXGTK_WIDGET_NONE
 } mixgtk_widget_id_t;
 
