@@ -1,9 +1,9 @@
 /* -*-c-*- -------------- xmixguile_cmd_dispatcher.c :
  * Implementation of the functions declared in xmixguile_cmd_dispatcher.h
  * ------------------------------------------------------------------
- * $Id: xmixguile_cmd_dispatcher.c,v 1.11 2003/06/02 23:22:31 jao Exp $
+ * $Id: xmixguile_cmd_dispatcher.c,v 1.12 2004/06/23 10:47:27 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -286,7 +286,7 @@ static SCM
 mix_src_name_ (void)
 {
   const gchar *path = mix_vm_cmd_dispatcher_get_src_file_path (vm_dispatcher_);
-  return gh_str02scm (path? g_basename (path) : "");
+  return gh_str02scm (path? g_path_get_basename (path) : "");
 }
 
 static SCM
@@ -300,7 +300,7 @@ static SCM
 mix_prog_name_ (void)
 {
   const gchar *path = mix_vm_cmd_dispatcher_get_program_path (vm_dispatcher_);
-  return gh_str02scm (path? g_basename (path) : "");
+  return gh_str02scm (path? g_path_get_basename (path) : "");
 }
 
 static SCM
