@@ -1,9 +1,9 @@
 /* -*-c-*- -------------- mix.c :
  * Implementation of the functions declared in mix.h
  * ------------------------------------------------------------------
- *  $Id: mix.c,v 1.4 2001/09/28 23:10:45 jao Exp $
+ *  $Id: mix.c,v 1.5 2002/04/10 23:39:40 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 2000, 2001, 2002 Free Software Foundation, Inc.
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,7 +81,7 @@ mix_stat_dir (const gchar *dirname, const gchar *alias)
     {
       if (errno != ENOENT || mkdir (dirname, S_IRWXU | S_IRWXG | S_IRWXO))
 	{
-	  g_warning ("Error creating %s dir %s: %s",
+	  g_warning (_("Error creating %s dir %s: %s"),
 		     alias, dirname, strerror (errno));
 	  return FALSE;
 	}
@@ -90,7 +90,7 @@ mix_stat_dir (const gchar *dirname, const gchar *alias)
   
   if (!(statbuf.st_mode & S_IFDIR))
     {
-      g_warning ("Error setting %s dir: %s is not a directory",
+      g_warning (_("Error setting %s dir: %s is not a directory"),
 		 alias, dirname);
       return FALSE;
     }
