@@ -1,7 +1,7 @@
 /* -*-c-*- -------------- mix_config.c :
  * Implementation of the functions declared in mix_config.h
  * ------------------------------------------------------------------
- *  $Id: mix_config.c,v 1.10 2004/06/07 12:18:22 jao Exp $
+ *  $Id: mix_config.c,v 1.11 2004/06/23 10:46:17 jao Exp $
  * ------------------------------------------------------------------
  * Copyright (C) 2001, 2002, 2004 Free Software Foundation, Inc.
  *
@@ -94,7 +94,7 @@ mix_config_new (const gchar *dirname, const gchar *filename)
     }
 
   autosave = mix_config_get (result, AUTOSAVE_KEY_);
-  result->autosave = autosave && !g_strcasecmp (autosave, AUTOSAVE_YES_);
+  result->autosave = autosave && !g_ascii_strcasecmp (autosave, AUTOSAVE_YES_);
 
   return result;
 }
@@ -267,7 +267,7 @@ mix_config_set_history_file (mix_config_t *config, const gchar *path)
     }
   else
     {
-      gchar *base = g_dirname (config->filename);
+      gchar *base = g_path_get_dirname (config->filename);
       gchar *hf = g_strconcat (base, G_DIR_SEPARATOR_S, path, NULL);
       mix_config_update (config, HISTORY_KEY_, hf);
       g_free (hf);
