@@ -170,7 +170,7 @@ mix_parser_delete (mix_parser_t *parser)
   /* clear the GSList values of future_refs and its keys */
   g_hash_table_foreach (parser->future_refs, delete_list_vals_, NULL);
   /* clear the ins_node_'s of the ins tree */
-  g_tree_traverse (parser->ins_table, delete_tree_vals_, G_IN_ORDER, NULL);
+  g_tree_foreach (parser->ins_table, delete_tree_vals_, NULL);
   /* destroy the tree and hash tables */
   g_tree_destroy (parser->ins_table);
   mix_symbol_table_delete (parser->symbol_table);
@@ -183,15 +183,13 @@ mix_parser_delete (mix_parser_t *parser)
 }
 
 /* Compile a mix source file */
-static guint
+static void
 update_future_refs_value_ (mix_parser_t *parser, const gchar *name,
 			   mix_short_t value, gboolean remove)
 {
   GSList *list = NULL;
   gpointer *plist = (gpointer *)(&list);
   gpointer key;
-  guint line = 0;
-
 
   g_assert (parser != NULL && name != NULL);
   if ( g_hash_table_lookup_extended (parser->future_refs, name, &key, plist) )
@@ -211,8 +209,6 @@ update_future_refs_value_ (mix_parser_t *parser, const gchar *name,
 	  }
 	  mix_word_add_address (node->ins, value);
 	  g_tree_insert (parser->ins_table, tmp->data, (gpointer)node);
-          /* record the first line the ref appears */
-          if (line == 0 || line > node->lineno) line = node->lineno;
 	  tmp = g_slist_next (tmp);
 	}
       if (remove) {
@@ -221,7 +217,6 @@ update_future_refs_value_ (mix_parser_t *parser, const gchar *name,
       }
       g_slist_free (list);
     }
-  return line;
 }
 
 #define update_future_refs_(parser,name,rem) \
@@ -235,8 +230,8 @@ update_ls_ (gpointer symbol, gpointer value, gpointer parser)
   mix_parser_t *par = (mix_parser_t *) parser;
 
   mix_word_to_ins_uncheck (w, ins);
-  mix_parser_add_ins (par, &ins,
-                      update_future_refs_ (par, (const gchar *)symbol, TRUE));
+  update_future_refs_ (par, (const gchar *)symbol, TRUE);
+  mix_parser_add_ins (par, &ins, 0);
   par->loc_count++;
 }
 
@@ -250,7 +245,8 @@ undef_warning_ (gpointer symbol, gpointer value, gpointer data)
   mix_parser_log_error (parser, MIX_PERR_UNDEF_SYM, 0, name, TRUE);
 
   mix_word_to_ins_uncheck (MIX_WORD_ZERO, ins);
-  mix_parser_add_ins (parser, &ins, update_future_refs_ (parser, name, FALSE));
+  update_future_refs_ (parser, name, FALSE);
+  mix_parser_add_ins (parser, &ins, 0);
   mix_symbol_table_insert (parser->symbol_table, name,
                            mix_short_to_word_fast (parser->loc_count));
 
@@ -334,8 +330,7 @@ mix_parser_write_code (mix_parser_t *parser, const gchar *code_file,
   g_free (source_path);
 
   if (context.file == NULL) return MIX_PERR_NOOUT;
-  g_tree_traverse (parser->ins_table, write_code_,
-		   G_IN_ORDER, (gpointer)&context);
+  g_tree_foreach (parser->ins_table, write_code_, (gpointer)&context);
   mix_code_file_delete (context.file);
   return parser->status;
 }
@@ -415,8 +410,7 @@ mix_parser_write_listing (mix_parser_t *parser, const gchar *list_file)
   fputs ( _("Src     Address  Compiled word           Symbolic rep\n"),
 	  context.file);
   fputs (sep, context.file);
-  g_tree_traverse (parser->ins_table, write_listing_,
-                   G_IN_ORDER, (gpointer)(&context));
+  g_tree_foreach (parser->ins_table, write_listing_, (gpointer)(&context));
   fputs (sep, context.file);
   fprintf (context.file, _("\n*** Start address:\t%d\n*** End address:\t%d\n"),
 	   mix_short_magnitude (parser->start),
@@ -446,7 +440,7 @@ mix_parser_load_vm (const mix_parser_t *parser, mix_vm_t *vm)
   g_return_val_if_fail (vm != NULL, MIX_PERR_INTERNAL);
   g_return_val_if_fail (parser->status == MIX_PERR_OK, parser->status);
   mix_vm_reset (vm);
-  g_tree_traverse (parser->ins_table, load_vm_, G_IN_ORDER, (gpointer)vm);
+  g_tree_foreach (parser->ins_table, load_vm_, (gpointer)vm);
   mix_vm_set_start_addr (vm, parser->start);
   return parser->status;
 }
