@@ -2,9 +2,9 @@
  * This file contains internal declarations used in the implementation
  * of the mix_vm_t type.
  * ------------------------------------------------------------------
- *  $Id: xmix_vm.h,v 1.8 2003/01/12 21:05:01 jao Exp $
+ *  $Id: xmix_vm.h,v 1.9 2004/06/06 19:53:13 jao Exp $
  * ------------------------------------------------------------------
- * Copyright (C) 2000, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2000, 2002, 2003, 2004 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,6 +50,7 @@ struct mix_vm_t
   mix_cmpflag_t cmpflag;
   mix_short_t loc_count;
   mix_vm_status_t status;
+  mix_vm_error_t last_error;
   mix_device_t * devices[BD_NO_];
   mix_address_t start_addr;	/* start address of loaded file */
   GTree *line_table;		/* source line no -> address  */
@@ -86,6 +87,8 @@ enum { A_ = 0, X_, J_, I1_, I2_, I3_, I4_, I5_, I6_ };
 #define get_pred_list_(vm) (vm->pred_list)
 #define get_address_list_(vm) (vm->address_list)
 #define get_status_(vm) (vm->status)
+#define get_last_error_(vm) (vm->last_error)
+#define set_last_error_(vm,error) ((vm)->last_error = (error))
 
 #define set_reg_(vm,r,x)			\
 do {						\
