@@ -1,7 +1,7 @@
 /* -*-c-*- -------------- xmix_vm_handlers.c :
  * Implementation of the functions declared in xmix_vm_handlers.h
  * ------------------------------------------------------------------
- * $Id: xmix_vm_handlers.c,v 1.8 2004/06/06 19:54:11 jao Exp $
+ * $Id: xmix_vm_handlers.c,v 1.9 2004/06/23 10:46:17 jao Exp $
  * ------------------------------------------------------------------
  * Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
  *
@@ -1276,7 +1276,8 @@ cmd_pbt_ (mix_vm_cmd_dispatcher_t *dis, const gchar *arg)
   gint k = 0, address;
   guint line;
   const mix_src_file_t *file = mix_vm_get_src_file (dis->vm);
-  char *name = file ? g_basename (mix_src_file_get_path (file)) : NULL;
+  char *name =
+    file ? g_path_get_basename (mix_src_file_get_path (file)) : NULL;
 
   const GSList *add = mix_vm_get_backtrace (dis->vm);
   while (add && (no == 0 || k < no))
