@@ -1,7 +1,7 @@
 /* -*-c-*- -------------- mixasm.c:
  * Main function of mixasm, the mix assembler
  * ------------------------------------------------------------------
- *  $Id: mixasm.c,v 1.5 2004/06/07 17:21:09 jao Exp $
+ *  $Id: mixasm.c,v 1.6 2004/07/30 12:31:04 jao Exp $
  * ------------------------------------------------------------------
  * Copyright (C) 2000, 2001, 2002, 2004 Free Software Foundation, Inc.
  *
@@ -40,7 +40,7 @@ enum {
   USAGE_OPT = 'u',
   OUT_OPT =  'o',
   LIST_OPT = 'l',
-  DEBUG_OPT = 'g'
+  NDEBUG_OPT = 'O'
 };
 
 
@@ -51,13 +51,13 @@ static struct option long_options_[] =
   {"usage", no_argument, 0, USAGE_OPT},
   {"output", required_argument, 0, OUT_OPT},
   {"list", optional_argument, 0, VER_OPT},
-  {"debug", no_argument, 0, DEBUG_OPT},
+  {"ndebug", no_argument, 0, NDEBUG_OPT},
   {0, 0, 0, 0}
 };
 
 static const gchar *USAGE_ =
-N_("Usage: %s [-vhulg] [-o OUTPUT_FILE] [--version] [--help]\n"
-   "\t[--usage] [--debug] [--output=OUTPUT_FILE] [--list[=LIST_FILE]] file\n");
+N_("Usage: %s [-vhulO] [-o OUTPUT_FILE] [--version] [--help]\n"
+   "\t[--usage] [--ndebug] [--output=OUTPUT_FILE] [--list[=LIST_FILE]] file\n");
 
 
 int
@@ -66,7 +66,7 @@ main (int argc, char **argv)
   int c;
   const char *prog_name = argv[0];
   const char *src = NULL, *out = NULL, *list = NULL;
-  gboolean use_list = FALSE, debug = FALSE;
+  gboolean use_list = FALSE, debug = TRUE;
 
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -95,8 +95,8 @@ main (int argc, char **argv)
 	  use_list = TRUE;
 	  list = optarg;
 	  break;
-	case DEBUG_OPT:
-	  debug = TRUE;
+	case NDEBUG_OPT:
+	  debug = FALSE;
 	  break;
      	case '?':
 	  /* getopt already handles the output of a warning message */
